# -*- coding: utf-8 -*-
"""
 --------------------
 Programming L. Seropian
 U1028, Inserm, CNRS property
 Lyon Neuroscience Research Center
==============================
Pupil dilation analysis (SMI RED 250)
Normalize and tidy preprocessed pupil data into pandas dataframe
=============================== 
-------------------------------

Copyright (C) 2019  SEROPIAN MOULIN BIDET-CAULET

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import os, sys, glob
import numpy as np
import pandas as pd
import scipy.stats as stats
import time

data = 'data' #path to preprocessed data
subjs = os.listdir(data) #subjects list
no_need= [] #excluded sujects

"""
Normalization methods:
- raw: preprocessed data only
- bsl_corr_only: baseline corrected data (see Zekveld et al. 2010)
- prop_change: diameters expressed as proportional change from baseline (see Wierda et al. 2012)
- wtms: data normalized using Within-trial mean scaling (see Kuchinsky et al. 2013)
- gms: data normalized using Grand mean scaling (see Engelhardt et al. 2010)
- zscore: pupil diameters expressed as Z-score (see McCloy et al. 2016)

"""
norms = ['raw','bsl_corr_only','prop_change','wtms','gms','zscore'] #normalization methods

"""
Baseline periods:
- baselineA (Start) onset: Start (start of trial)
- baselineB (StartCue) onset: Beep (first auditory cue: 'bip' in French)
- baselineC (EndCue) onset: end of Beep (end of first auditory cue)
- baselineD (StartWord) offset: Beginning of word ('mot' in French)

"""

baselines = ['baselineA','baselineB','baselineC','baselineD'] #baseline periods

"""
Baseline durations:
- 500 ms
- 1000 ms

"""

baselines_len = [125, 250] #baseline durations in samples: 500ms, 1s 

nb_files = 8 #number of file per subject (1 per block)
first_trial = 3 #pass 1 to analyse 12 words (whole block), pass 3 to analyse 10 words

sub_eye_ok = pd.read_csv('data/subj1_2_eye.csv') #specific eye to analyse per subject

##### Enter specific subjects to analyze here #####
#subjs = []
#####

for subj in subjs:
    if subj.endswith('.csv'): #select only subject data folder
        continue
    elif subj in no_need:
        continue
    else:
        
        print(subj)
        eye = sub_eye_ok[sub_eye_ok['subject']==subj].eye.iloc[0] #select clean eye 
        print(eye)

        if eye == 'R':
            row = 0 #select corresponding data (Right eye)
        else:
            row = 14 #select corresponding data (left eye)

        df_pupildat_liss = pd.DataFrame()
        temp_file=[]
        for norm in norms:
            print(norm)
            
            for bsl in baselines:
                print(bsl)
                if norm == 'raw' and bsl != 'baselineA': #no baseline period needed for raw data
                    continue
                
                for bsl_l in baselines_len:
                    if norm=='raw' and bsl_l == 250: #no baseline duration needed for raw data
                        continue 
                    
                    print(bsl_l)
                    
                    for nb in range(0, nb_files):
                        try:
                            pretrial = 250 #in samples: 1s pretrial
                            print()
                            print()
                            print('file ' + str(format(nb+1, '02d')))
                            path= data + '/' + subj
                            fileid_details = glob.glob(path + '/csv_files/' + subj + str(format(nb+1, '02d')) + '*' + 'Samplesjul17tttAdetails.csv')[0] #find corresponding file
                            print(fileid_details)
                            #Read block details 
                            details=pd.read_csv(fileid_details)

                            cond = details['condition'].iloc[0] #find condition

                            fileid_boundaries = glob.glob(path + '/csv_files/' + subj + str(format(nb+1, '02d')) + '*' + 'Samplesjul17tttAlim.csv')[0] #find corresponding file
                            print(fileid_boundaries)
                            #Read trial boundaries infos
                            boundaries=pd.read_csv(fileid_boundaries)

                            fileid_data_liss = glob.glob(path + '/csv_files/' + subj + str(format(nb+1, '02d')) + '*' + 'Samplesjul17tttAsamps9liss.csv')[0] #find corresponding file
                            print(fileid_data_liss)
                            #Read preprocessed data
                            data_liss = pd.read_csv(fileid_data_liss)

                            fileid_man = glob.glob(path + '/csv_files/' + subj + str(format(nb+1, '02d')) + '*' + 'SamplestttSMIBman.csv')[0]
                            print(fileid_man)
                            #Read manually included trials infos
                            man = pd.read_csv(fileid_man)

                            inclus = man['inclumanu'] #manual inclusion
                            correct = man['mot correct'] #word repeated correclty or not
                            syll1 = man['syll1'] #first syllable repeated correclty or not
                            syll2 = man['syll2'] #second syllable repeated correclty or not
                            pho1 = man['pho1'] #first phoneme repeated correclty or not
                            pho2 = man['pho2'] #second phoneme repeated correclty or not
                            pho3 = man['pho3'] #third phoneme repeated correclty or not
                            pho4 = man['pho4'] #fourth phoneme repeated correclty or not
                            pho5 = man['pho5'] #fifth phoneme repeated correclty or not
                            pho6 = man['pho6'] #sixth phoneme repeated correclty or not
                            repetot = man['repetot'] #word repeated early or not
                            pourcbadsample = man['pourcbadsample'] #percentage of bad samples in trial

                            if subj == 'subj2' and (cond=='AS' or cond=='SS'): #no pretrial recording for this subject
                                pretrial = 0

                            if (nb+1) % 2 != 0: #odd files = first file per condition
                                nb_included_trials = 0
                            
                            ##### Compute gms
                            if norm == 'gms':
                                # grand mean time series (without repetition word part)
                                grand_ts = []
                                for i in range(first_trial, 13): #last word = word nb12
                                    start_o = boundaries['start'].iloc[i]
                                    start = data_liss.index[data_liss.iloc[:,0] == np.int64(start_o)+1].tolist()[0] #find actual index for beginning
                                    blop_o = boundaries['blop'].iloc[i]
                                    blop = data_liss.index[data_liss.iloc[:,0] == np.int64(blop_o)+1].tolist()[0] #find actual index for blop
    
                                    lower=np.int64(start)
                                    upper=np.int64(blop)

                                    ts=data_liss[eye + ' Mapped Diameter [mm]'].iloc[lower:upper]
                                    trial=ts.values
                                    grand_ts.extend(trial)

                                grand_mean_ts = np.mean(grand_ts)
                            #####
                            
                            for trial in range(first_trial,13): #last word = word nb12
                                if inclus[trial+row] == 1 and pourcbadsample[trial+row]<60 and repetot[trial] == 0: #trial included in analysis + enough good samples + repeated after blop
                                    print('trial ' + str(trial) + ' OK')
                                    nb_included_trials += 1
                                    temp_trial=pd.DataFrame()

                                    #Find index for trial boundaries
                                    start_o = boundaries['start'].iloc[trial]
                                    start = data_liss.index[data_liss.iloc[:,0] == np.int64(start_o)+1].tolist()[0] #find actual index for beginning
                                    bip_o = boundaries['bip'].iloc[trial]
                                    bip = data_liss.index[data_liss.iloc[:,0] == np.int64(bip_o)+1].tolist()[0] #find actual index for beep
                                    word_o = boundaries['mot'].iloc[trial]
                                    word = data_liss.index[data_liss.iloc[:,0] == np.int64(word_o)+1].tolist()[0] #find actual index for word
                                    blop_o = boundaries['blop'].iloc[trial]
                                    blop = data_liss.index[data_liss.iloc[:,0] == np.int64(blop_o)+1].tolist()[0] #find actual index for blop
                                    stop_o = boundaries['start'].iloc[trial+1]
                                    stop = data_liss.index[data_liss.iloc[:,0] == np.int64(stop_o)+1].tolist()[0] #find actual index for end

                                    #trial parts
                                    startbip = np.int64(bip-start)
                                    bipword = np.int64(word-bip)
                                    wordblop = np.int64(blop-word)
                                    blopstop = np.int64(stop-blop)

                                    if norm != 'raw':

                                        ##### Compute wtms
                                        if norm == 'wtms':
                                            lo = np.int64(start)
                                            up = np.int64(blop)
                                            #mean trial time series for each trial (without repetition word part)
                                            mean_trial_ts = data_liss[eye + ' Mapped Diameter [mm]'].iloc[lo:up].mean()
                                            #each time point divided by mean
                                            new_trial_ts = data_liss[eye + ' Mapped Diameter [mm]'].iloc[np.int64(start)-pretrial:np.int64(stop)]/mean_trial_ts
                                        #####

                                        #####Baseline boundaries

                                        if bsl == 'baselineA':
                                            lower = np.int64(start)
                                            upper = lower + bsl_l

                                        if bsl == 'baselineB':
                                            lower = np.int64(bip)
                                            upper = lower + bsl_l

                                        if bsl == 'baselineC':
                                            lower = np.int64(bip)+100 #wait for end of bip sound (400ms)
                                            upper = lower + bsl_l

                                        if bsl == 'baselineD':
                                            upper = np.int64(word)
                                            lower = upper - bsl_l
                                        #####
                                        
                                        ##### Compute gms
                                        if norm == 'gms':
                                            #each time point divided by grand mean
                                            new_ts = data_liss[eye + ' Mapped Diameter [mm]'].iloc[np.int64(start)-pretrial:np.int64(stop)]/grand_mean_ts
                                        #####

                                        ##### Compute Z-score
                                        if norm == 'zscore':
                                            #compute z-score
                                            mean_win = data_liss[eye + ' Mapped Diameter [mm]'].iloc[np.int64(start):np.int64(blop)].mean() #mean of trial (without repetition word part)
                                            std_win = data_liss[eye + ' Mapped Diameter [mm]'].iloc[np.int64(start):np.int64(blop)].std(ddof=0) #std of trial (without repetition word part)
                                            ts_zscore = (data_liss[eye + ' Mapped Diameter [mm]'].iloc[np.int64(start)-pretrial:np.int64(stop)]-mean_win)/std_win
                                        #####

                                        ##### Compute baselines
                                        if norm == 'wtms':
                                            baseline = new_trial_ts.loc[lower:upper].mean() 
                                        elif norm == 'gms':
                                            baseline = new_ts.loc[lower:upper].mean()
                                        elif norm == 'zscore':
                                            baseline = ts_zscore.loc[lower:upper].mean()
                                            
                                        else:
                                            baseline = data_liss[eye + ' Mapped Diameter [mm]'].iloc[lower:upper].mean()
                                        #####

                                        ####Baseline correction
                                        if norm == 'bsl_corr_only':
                                            trial_liss = data_liss[eye + ' Mapped Diameter [mm]'].iloc[np.int64(start)-pretrial:np.int64(stop)] - baseline
                                        if norm == 'prop_change':
                                            trial_liss = (data_liss[eye + ' Mapped Diameter [mm]'].iloc[np.int64(start)-pretrial:np.int64(stop)] - baseline)/baseline
                                        if norm == 'wtms':
                                            trial_liss = new_trial_ts - baseline
                                        if norm == 'gms':
                                            trial_liss = new_ts - baseline
                                        if norm == 'zscore':
                                            trial_liss = ts_zscore - baseline
                                        #####

                                    else:
                                        #Preprocessed data (non normalized)
                                        trial_liss = data_liss[eye + ' Mapped Diameter [mm]'].iloc[np.int64(start)-pretrial:np.int64(stop)]
                                    
                                    # Trial parts    
                                    trial_part=[]
                                    trial_part[0:pretrial]=['pre-trial']*pretrial #1s pre-trial
                                    trial_part[pretrial:startbip+pretrial] = ['startbip']*startbip
                                    trial_part[startbip+pretrial:(startbip+bipword+pretrial)] = ['bipword']*bipword
                                    trial_part[(startbip+bipword+pretrial):(startbip+bipword+wordblop+pretrial)] = ['wordblop']*wordblop
                                    trial_part[(startbip+bipword+wordblop+pretrial):(startbip+bipword+wordblop+blopstop+pretrial)] = ['blopstop']*blopstop

                                    temp_trial['trial_part'] = trial_part

                                    # Responses
                                    if correct[trial] == 1:
                                        sdt = 'Hit'

                                    else:
                                        sdt = 'Miss'
                                        if pho1[trial]==1 or pho2[trial]==1 or pho3[trial]==1 or pho4[trial]==1 or pho5[trial]==1 or pho6[trial]==1:
                                            sdt = 'Pmiss'

                                    # Phonemes correctly repeated
                                    phos = [pho1[trial],pho2[trial],pho3[trial],pho4[trial],pho5[trial],pho6[trial]]
                                    pho_ok = phos.count(1)
                                    pho_pas_ok = phos.count(0)
                                    nb_pho = pho_ok + pho_pas_ok

                                    prc_pho = pho_ok*1.0/((nb_pho)*1.0)

                                    # Time vector
                                    time_liss = []
                                    for t in range(-pretrial,0):
                                        time_liss.append(4*t)
                                    for t in range(0, len(trial_liss)-pretrial):
                                        time_liss.append(4*t)
                                    
                                    temp_trial['time'] = time_liss

                                    #Adjust trial number
                                    if (nb+1) % 2 == 0: #first file with the condition or not
                                        temp_trial['trial'] = trial + 12
                                    else:
                                        temp_trial['trial'] = trial

                                    temp_trial['subject'] = subj #subject code
                                    temp_trial['condition'] = cond #condition
                                    
                                    #Pupil dilation values
                                    if norm == 'zscore':
                                        temp_trial['pupil_size'] = trial_liss.values
                                    else:
                                        temp_trial['pupil_size'] = trial_liss.values
                                    
                                    temp_trial['normalization'] = norm #normalization methods
                                    
                                    if norm == 'raw':
                                        temp_trial['baseline'] = 'noneed' #no baseline period if raw
                                        temp_trial['baseline_len'] = 0 #no baseline duration if raw
                                    else:
                                        temp_trial['baseline'] = bsl #baseline period
                                        temp_trial['baseline_len'] = np.int64(bsl_l) #baseline duration
                                    
                                    #Other trial infos
                                    temp_trial['sdt'] = sdt
                                    temp_trial['nb_included_trials'] = nb_included_trials
                                    temp_trial['nb_phos'] = nb_pho
                                    temp_trial['prc_phos'] = prc_pho
                                    temp_trial['sy1'] = syll1[trial]
                                    temp_trial['sy2'] = syll2[trial]
                                    temp_trial['tot_sy'] = (syll1[trial] + syll2[trial])/2
                                    temp_trial['correct_word'] = correct[trial]
                                    temp_file.append(temp_trial)
                     
   
                        except:
                            print(subj + ' '  + str(nb) + ' - No data for this file number') #excluded condition
                            continue

    if not temp_file: #no data included for subject
        continue   
    start_time = time.time()
    #Huge tidy DataFrame
    df_pupildat_liss = pd.concat(temp_file,axis=0,ignore_index=True)
    print(df_pupildat_liss)
    #Save Dataframe
    df_pupildat_liss.to_csv(subj +'_df_norm_pupil_data_' + eye +'_10words.csv',
        index=False)
    print("final file %s seconds ---" % (time.time() - start_time))        
    print(subj + ' DONE')
    print()

print('TIDY DATA DONE') 

